"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongooseRepository = void 0;
const common_1 = require("@nestjs/common");
const mongoose_helper_1 = require("./mongoose.helper");
const mongoose_1 = require("@nestjs/mongoose");
const microservices_exceptions_1 = require("@devseeder/microservices-exceptions");
class MongooseRepository {
    constructor(model) {
        this.model = model;
        this.logger = new common_1.Logger(this.constructor.name);
    }
    insertOne(item, name, session = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.create(item, session).then((savedDoc) => {
                this.logger.log(`${item.constructor.name} '${name}' saved successfully!`);
                this.logger.log(`ID: ${savedDoc[0]._id}`);
                return savedDoc[0]._id;
            }, (err) => {
                this.logger.error(err.message);
                throw new microservices_exceptions_1.MongoDBException(err.message, err.code);
            });
        });
    }
    create(document, session = null) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                this.model.create([document], this.getSessionOptions(session), function (err, savedDoc) {
                    if (err)
                        reject(err);
                    resolve(savedDoc);
                });
            }));
        });
    }
    groupBy(group, match = {}, select = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateParams = [];
            if (Object.keys(match).length > 0)
                aggregateParams.push({ $match: match });
            aggregateParams.push({
                $group: Object.assign({ _id: group, count: { $sum: 1 } }, mongoose_helper_1.MongooseHelper.buildSelectAggregated(select)),
            });
            return this.model.aggregate(aggregateParams);
        });
    }
    aggregate(group, match = {}, select = {}, unwind = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateParams = [];
            if (Object.keys(match).length > 0)
                aggregateParams.push({ $match: match });
            if (unwind.length > 0)
                aggregateParams.push({ $unwind: `\$${unwind}` });
            aggregateParams.push({
                $group: Object.assign(Object.assign(Object.assign({}, group), { count: { $sum: 1 } }), select),
            });
            return this.model.aggregate(aggregateParams);
        });
    }
    aggregateNotExists(from, joinFrom, joinLet, match = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateParams = [];
            aggregateParams.push(mongoose_helper_1.MongooseHelper.buildLookupAggregate(from, joinFrom, joinLet, match));
            const matchParam = {};
            matchParam[`aggElement.${joinFrom}`] = { $exists: false };
            if (Object.keys(match).length > 0)
                aggregateParams.push({ $match: Object.assign(Object.assign({}, matchParam), match) });
            aggregateParams.push({
                $project: {
                    likes: false,
                },
            });
            return this.model.aggregate(aggregateParams);
        });
    }
    findAll(select = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Object.keys(select).length === 0)
                select = { _id: 0 };
            return this.model.find({}).select(select).lean().exec();
        });
    }
    findById(id, select = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Object.keys(select).length === 0)
                select = { _id: 0 };
            return this.model.findById(id).select(select).lean().exec();
        });
    }
    updateOneById(id, data, session = null, pushData = {}, strict = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.updateOne({ _id: id }, data, pushData, strict, session);
            this.logger.log(`${(0, mongoose_1.getModelToken)(this.model.name)} ${id} - Succesfully updated!.`);
            return res;
        });
    }
    updateOne(query, data, pushData = {}, strict = false, session = null) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = Object.assign({ upsert: false, strict }, this.getSessionOptions(session));
            this.logger.log(`Update Options: ${options}`);
            yield this.model
                .findOneAndUpdate(query, Object.assign({ $set: data }, pushData), options)
                .catch((err) => {
                throw new microservices_exceptions_1.MongoDBException(err.message, err.code);
            });
        });
    }
    updateMany(query, data, pushData = {}, strict = true, session = null) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = this.model.updateMany(query, Object.assign({ $set: data }, pushData), Object.assign({ upsert: false, strict }, this.getSessionOptions(session)));
            return result.exec((err, result) => {
                if (err)
                    throw new microservices_exceptions_1.MongoDBException(err.message, err.name);
                return result;
            });
        });
    }
    deleteOneById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.model.deleteOne({ id });
        });
    }
    find(searchParams, select = {}, sort = {}, regex = true, limit = 0, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Object.keys(select).length === 0)
                select = { _id: 0 };
            searchParams = regex
                ? mongoose_helper_1.MongooseHelper.buildRegexFilterQuery(searchParams)
                : searchParams;
            let res = this.model.find(searchParams);
            if (skip)
                res = res.skip(skip - 1);
            if (limit)
                res = res.limit(limit);
            if (typeof sort === 'object' && Object.keys(sort).length > 0)
                res = res.sort(sort);
            if (typeof sort === 'object' && Object.keys(sort).length > 0)
                res = res.sort(sort);
            return res.select(select).lean().exec();
        });
    }
    getIndexes() {
        return this.model.collection.getIndexes();
    }
    count(searchParams) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.model.countDocuments(searchParams);
        });
    }
    startSession() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.session)
                this.logger.warn(`Session already exists`);
            this.session = yield this.model.startSession();
        });
    }
    endSession() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.session)
                this.logger.warn(`Session already finished`);
            yield this.session.endSession();
            this.session = null;
        });
    }
    startTransaction() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.session)
                return;
            this.logger.log(`Starting transaction...`);
            yield this.startSession();
            yield this.session.startTransaction();
            return this.session;
        });
    }
    commit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.session)
                return;
            this.logger.log(`Committing transaction...`);
            yield this.session.commitTransaction();
            yield this.endSession();
            this.logger.log(`Transaction commited.`);
        });
    }
    rollback() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.session)
                return;
            this.logger.log(`Starting Rollback...`);
            yield this.session.abortTransaction();
            yield this.endSession();
            this.logger.log(`Rollback finished.`);
        });
    }
    getSessionOptions(session = null) {
        const clientSession = session
            ? session
            : this.session
                ? this.session
                : null;
        return clientSession ? { session: clientSession } : {};
    }
}
exports.MongooseRepository = MongooseRepository;
